// lib/transformer.js

'use strict';

var parts = function (/* args */) {
  var content = [];

  var addArgs = function (args) {
    for (var i = 0; i < args.length; i++) {
      content.push(args[i]);
    }
  };

  addArgs(arguments);

  return {
    push: function (/* args */) {
      addArgs(arguments);
    },
    toString: function () {
      return content.join('');
    }
  };
};

var dummyTransformer = function () {
  var outputContent = parts();

  var rawDump = function (obj) {
    outputContent.push(obj.raw);
  };

  var voidFn = function () {};

  var add = function (/* args */) {
    for (var i = 0; i < arguments.length; i++) {
      outputContent.push(arguments[i]);
    }
  };

  return {
    onDoctype: rawDump,
    onOpenTag: rawDump,
    onCloseTag: rawDump,
    onText: rawDump,
    onComment: rawDump,
    onError: voidFn,
    onWarning: voidFn,

    push: add,

    output: function () {
      return outputContent.toString();
    }
  };
};

var baseTransformer = function () {
  var isLexemEmpty = function (lexem) {
    return typeof lexem.content !== 'string';
  };

  var openTag = function (lexem) {
    var pp = parts('<', lexem.name);
    var attrs = lexem.attrs;
    Object.keys(attrs).forEach(function (key) {
      pp.push(' ', key, '="' + attrs[key] + '"');
    });
    if (isLexemEmpty(lexem)) {
      pp.push('/');
    }
    pp.push('>');
    return pp.toString();
  };

  var closeTag = function (lexem) {
    if (isLexemEmpty(lexem)) {
      return '';
    } else {
      return parts('</', lexem.name, '>').toString();
    }
  };

  var text = function (string) {
    return string || '';
  };

  var tag = function (name, attrs, content) {
    if (typeof attrs === 'string' && typeof content === 'undefined') {
      content = attrs;
      attrs = null;
    }

    if (!attrs) {
      attrs = {};
    }

    var lexem = {
      name: name,
      attrs: attrs,
      content: content
    };

    return (
      openTag(lexem) + text(content) + closeTag(lexem)
    );
  };

  var passLexem = function (lexem) {
    this.push(lexem.raw);
  };

  return Object.assign(
    dummyTransformer(), {
      tag: tag,
      pass: passLexem
    }
  );
};

var transformer = function (methods) {
  if (typeof methods === 'undefined') {
    methods = {};
  }

  return Object.assign(baseTransformer(), methods);
};

exports.transformer = transformer;

// var strictTransformer = function (_options) {
//   var outputContent = parts();

//   var onDoctype = function (obj) {
//     var pp = parts('<!DOCTYPE ', obj.content, '>');

//     outputContent.push(pp.toString());
//   };

//   var onOpenTag = function (obj) {
//     var pp = parts('<', obj.name);
//     var attrs = obj.attrs || {};
//     Object.keys(attrs).forEach(function (key) {
//       if (attrs[key] === null) {
//         pp.push(' ', key);
//       } else {
//         pp.push(' ', key, '="', attrs[key], '"');
//       }
//     });
//     if (obj.forceEmpty) {
//       pp.push('/');
//     }
//     pp.push('>');

//     outputContent.push(pp.toString());
//   };

//   var onCloseTag = function (obj) {
//     var pp = parts('</', obj.name, '>');

//     outputContent.push(pp.toString());
//   };

//   var onText = function (obj) {
//     outputContent.push(obj.content);
//   };

//   var onComment = function (obj) {
//     var pp = parts('<!-- ', obj.content, ' -->');
//     outputContent.push(pp.toString());
//   };

//   return {
//     onOpenTag: onOpenTag,
//     onCloseTag: onCloseTag,
//     onText: onText,
//     onComment: onComment,
//     output: function () {
//       return outputContent.toString();
//     }
//   };
// };
